
#include <string>
#include <cstdio>

#include "as_config.h"

#ifdef POSIX_TERMIOS

// Posix headers
#include <termios.h>
#include <unistd.h>

termios new_attributes;
termios saved_attributes;

#elif HAVE_GETCH

extern "C" {int getch();}

#endif

void reset_input_mode (void) {
#ifdef POSIX_TERMIOS
  tcsetattr (STDIN_FILENO, TCSANOW, &saved_attributes);
#endif
}

void init_termios() {
#ifdef POSIX_TERMIOS
  if (!isatty (STDIN_FILENO)) {
    puts("Error: Stdin not a terminal.");
    exit (-1);
  }

  //
  // Save the terminal attributes so we can restore them later.
  //
  tcgetattr (STDIN_FILENO, &saved_attributes);
  atexit(reset_input_mode);

  //
  // Set up the terminal to read in a line character at a time
  //
  tcgetattr (STDIN_FILENO, &new_attributes);
  new_attributes.c_lflag &= ~(ICANON); // Clear ICANON 
  new_attributes.c_cc[VMIN] = 1;
  new_attributes.c_cc[VTIME] = 0;
  tcsetattr (STDIN_FILENO, TCSAFLUSH, &new_attributes);
#endif
}

void tgetline(string & line) {
#ifdef POSIX_TERMIOS
  tcsetattr (STDIN_FILENO, TCSANOW, &saved_attributes);
#endif
  line.resize(0);
  char c;
  while ((c = getchar()) != '\n')
    line += c;
#ifdef POSIX_TERMIOS
  tcsetattr (STDIN_FILENO, TCSANOW, &new_attributes);
#endif
}

char tget() {
#ifdef POSIX_TERMIOS
  char c;
  read (STDIN_FILENO, &c, 1);
  return c;
#elif HAVE_GETCH
  char c = getch();
  putchar(c);
  return c;
#else
  char c = getchar();
  char d = c;
  while (d != '\n') d = getchar();
  return c;
#endif
}

